#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_metricz
----------------------------------

Tests for `metricz` module.
"""

import pytest


from metricz import metricz


class TestMetricz(object):

    @classmethod
    def setup_class(cls):
        pass

    def test_something(self):
        pass

    @classmethod
    def teardown_class(cls):
        pass
